#ifndef  _ISING_H_
#define  _ISING_H_

#include "random.h"

/************************************************************************
 * Structures                                                           *
 ************************************************************************/

typedef struct
{
    int i;
    int j;
    double J; /* coupling */
} ising_edge;

typedef struct
{
    int *NtimesEss; /* sum(s_j,s_i) length e */
    int *NtimesEs;  /* sum(s_i) length n */
    int N; /* number of data points */
} ising_data;

typedef enum { UP=+1, DOWN=-1, JOKER=0 } spin;
typedef struct
{
    double beta; /* global temp */

    int n; /* number of nodes  */
    int e; /* number of edges  */
    int rows, cols, type; /* so plot program knows if/how to render nodes */
    ising_edge *edges;
    double *h; /* biases */
    spin *spins;
    int *neigh; /* length=(2e+n): (num_ne, idxedge1, idxedge2, ...) X n */
    int *neighidx; /* length=(n): points into neigh for each node for random access */
    int *ptr;       /* clustering data see arXiv:cond-mat/0101295 v2 */
} ising_model;

/* p(s) \propto 1/Z(J,h) \exp( \sum_{(i,j)} J_{i,j} s_i s_j + \sum_i s_i h_i )
 */

/************************************************************************
 * Function prototypes                                                  *
 ************************************************************************/
extern const char *wrap_text;
ising_model* ising_make(int rows, int cols, int type);
ising_model* ising_clone(const ising_model* from);
void ising_copy_colors(const ising_model* from, ising_model *to);
void ising_free(ising_model *ising);
void ising_conditional(ising_model* ising, int node, double* up_prob);

/* I wanted to inline these but had some compiler trouble */
spin ising_get_spin(const ising_model* ising, int node);
void ising_set_spin(ising_model* ising, int node, spin value);


void ising_read_in_spins(ising_model* ising, const char* filename);
void ising_write_out_spins(const ising_model* ising, const char* filename);

double find_E(ising_model* G);

/*
void free_data(ising_data* data);
int summary_gibbs_move(ising_model* G);
void random_spins(ising_model* G);
double find_E(ising_model* G);
double find_E_data(ising_data* data, ising_model* G);
double log_prior(ising_model* G);
void alloc_data(ising_data* data, const ising_model *G);
void clear_data(ising_data* data, const ising_model *G);
void add_spins(ising_data* data, const ising_model *G);
void load_data(ising_data* data, const ising_model *G, const char* dfname);
*/

#endif /* #ifndef  _ISING_H_  */
