#include <stdlib.h>
#include <stdio.h>
#include <float.h>
#include <math.h>

#include "mavm_clamp.h"
#include "ising_summary.h"

int GLOBAL_mc_counter = 0;
double GLOBAL_step_size;
int which = 0; 

void ising_tr_function(void* summary_state, double* nums)
{
    int i;
    double max_up_prob, min_up_prob;
    ising_model* ising = (ising_model*) summary_state;

    for (i=0; i < ising->n; ++i) {
        ising_summary_conditional(ising, i, &max_up_prob, &min_up_prob);
        if (nums[i] < min_up_prob)
            ising_set_spin(ising, i, UP);
        else if (nums[i] > max_up_prob)
            ising_set_spin(ising, i, DOWN);
        else
            ising_set_spin(ising, i, JOKER);
    }
    GLOBAL_mc_counter++;
}


double ising_prior_fn(void* model)
{
    ising_model* ising = (ising_model*) model;

    double J = ising->edges[0].J;
    double h = ising->h[0];
    if ((J<0) || (J>0.4) || (h>1) || (h<-1)) {
        /* printf("prior vetoing %g %g\n",h,J); */
        return -DBL_MAX;
    } else
        return 0;
}


double ising_prop_fn(void* prop_model, void* model, Rand_t seed)
{
    double delta;
    int i;
    ising_model* from = (ising_model*) model;
    ising_model* to = (ising_model*) prop_model;

    delta = GLOBAL_step_size * Rangauss(seed);

    which = (!which);
    if (which) {
        /* perturb bias, copy couplings */
        for (i=0; i < from->n; i++)
            to->h[i] = from->h[i] + delta;
        for (i=0; i < from->e; i++)
            to->edges[i].J = from->edges[i].J;
    } else {
        /* perturb couplings, copy bias */
        for (i=0; i < from->n; i++)
            to->h[i] = from->h[i];
        for (i=0; i < from->e; i++)
            to->edges[i].J = from->edges[i].J + delta;
    }
    return 0.0;
}


void ising_slice_set_x(double xx, void* model)
{
    int i;
    ising_model* ising = (ising_model*) model;

    if (which) {
        /* set bias */
        for (i=0; i < ising->n; i++)
            ising->h[i] = xx;
    } else {
        /* set couplings */
        for (i=0; i < ising->e; i++)
            ising->edges[i].J = xx;
    }
}


double ising_slice_get_x(void* model)
{
    ising_model* ising = (ising_model*) model;

    /* When this function is called, toggle which variable should be updated and
     * tell slice sampling value of param about to be updated. */
    which = (!which);
    if (which) {
        return ising->h[0];
    } else {
        return ising->edges[0].J;
    }
}


void ising_report_fn(void* model)
{
    if (!which) {
        ising_model* ising = (ising_model*) model;
        printf("%d %g %g\n", GLOBAL_mc_counter, ising->h[0], ising->edges[0].J);
        GLOBAL_mc_counter = 0;
    }
}


/* lambda=0 picks second G, lambda=1 picks first G */
void ising_param_mix(ising_model* Gmid, ising_model* G, ising_model* Gpoint, double lambda)
{
	int i;
	for (i=0; i < Gmid->n; i++)
		Gmid->h[i] = lambda*(G->h[i]) + (1-lambda)*(Gpoint->h[i]);
	for (i=0; i < Gmid->e; i++)
		Gmid->edges[i].J = lambda*(G->edges[i].J) + (1-lambda)*(Gpoint->edges[i].J);
	/* fprintf(stdout,"PARAMS: %g %g\n",Gmid->h[0],Gmid->edges[0].J); */
}

