#include "ising_summary.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
/* #include "cftp.h" */

void ising_summary_conditional(ising_model* G, int node, double* max_up_prob, double* min_up_prob)
{
    int i;
    int *spins=G->spins;
    int *neigh=G->neigh;
    ising_edge *edges=G->edges;
    ising_edge edge;
    int idx = G->neighidx[node];
    int num_neigh=neigh[idx];
    double max_Lpstar_up = 0;
    double min_Lpstar_up = 0;
    spin cur_setting = ising_get_spin(G, node);

    ising_set_spin(G, node, UP);
    max_Lpstar_up += G->h[node];
    min_Lpstar_up += G->h[node];
    for(i=0; i<num_neigh; ++i) {
        idx++;
        edge=edges[neigh[idx]];
        switch ((spins[edge.i])*(spins[edge.j])) {
        case 1:
            max_Lpstar_up += edge.J;
            min_Lpstar_up += edge.J;
            break;
        case -1:
            max_Lpstar_up -= edge.J;
            min_Lpstar_up -= edge.J;
            break;
        case 0:
            max_Lpstar_up += fabs(edge.J);
            min_Lpstar_up -= fabs(edge.J);
            break;
        default:
            abort();
        }
    }
    ising_set_spin(G, node, cur_setting);

    (*max_up_prob) = 1.0/(1.0+exp(-2*G->beta*max_Lpstar_up));
    (*min_up_prob) = 1.0/(1.0+exp(-2*G->beta*min_Lpstar_up));
    /* printf("max %g   min %g\n",*max_up_prob,*min_up_prob); */
    /* assert(min_Lpstar_up <= max_Lpstar_up);
    assert(*min_up_prob <= *max_up_prob); */
} 

void ising_ss_blanker(void* summary_state)
{
    int i;
    ising_model* ising = (ising_model*) summary_state;
    
    for (i=0; i < ising->n; ++i)
        ising_set_spin(ising, i, JOKER);
}

int ising_ss_coalesced(void* summary_state)
{
    int i;
    ising_model* ising = (ising_model*) summary_state;

    for (i=0; i < ising->n; ++i) {
        if (ising_get_spin(ising, i) == JOKER)
            return 0;
    }
    return 1;
}

ss_match ising_ss_equality(const void* state, const void* summary_state)
{
    int i;
    ss_match match;
    ising_model* concrete = (ising_model*) state;
    ising_model* ss = (ising_model*) summary_state;

    match = TRUE;
    for (i=0; i < concrete->n; ++i) {
        if (ising_get_spin(ss, i) == JOKER) {
            match = MAYBE;
            continue;
        }
        if (ising_get_spin(ss, i) != ising_get_spin(concrete, i)) {
            match = FALSE;
            break;
        }
    }
    return match;
}


void ising_summary_sweep(void* state, Rand_t seed)
{
    int i;
    double max_up_prob, min_up_prob;
    double u_r;
    ising_model* ising = (ising_model*) state;

    for (i=0; i < ising->n; ++i) {
        ising_summary_conditional(ising, i, &max_up_prob, &min_up_prob);
        u_r = Randouble(seed);
        if (u_r < min_up_prob)
            ising_set_spin(ising, i, UP);
        else if (u_r > max_up_prob)
            ising_set_spin(ising, i, DOWN);
        else
            ising_set_spin(ising, i, JOKER);
    }
}


/* 
void ising_cftp(ising_model* ising, Rand_t seed)
{
    cftp(ising, ising_ss_blanker, ising_summary_sweep, ising_ss_coalesced, seed);
}
*/
