#ifndef  _MAVM_CLAMP_H_
#define  _MAVM_CLAMP_H_

#include "mc_operators.h"
#include <stdlib.h>

extern double GLOBAL_step_size;

/* To provide log-likelihood and log-prior functions */
typedef double (*model_fn)(void* model);

typedef void (*log_fn)(void* model);

/* Set params of prop_model to be perturbation of models.
 * Return log prob ratio of proposal to add on to log MH ratio (zero if symmetric) */
typedef double (*prop_fn)(void* prop_model, void* model, Rand_t seed);

typedef double (*slice_fn)(void* model, void* prop_model, double* Lfh, Rand_t seed);

typedef void (*slice_set_x_fn)(double xx, void* model);
typedef double (*slice_get_x_fn)(void* model);

/* Mix params of models for tempering steps */
/* "mixed_model = lambda*model1 + (1-lambda)*model2" */
typedef void (*mix_fn)(void* mixed_model, void* model1, void* model2, double lambda);

typedef struct
{
    void* model;       /* Should have data and current params. */
    void* prop_model;  /* Should have pointer to same data, will be used to evaluate alternative params and swapped with model. */
    void* fit_model;   /* Has fixed params fitted to real data. This model's data will be internally altered. */
    void* dummy_model; /* Used internally, params and data will be altered. */
    int N_data;        /* Number of real data points (and so fantasies to draw) */
    prop_fn prop_params;
    slice_set_x_fn slice_set_x;
    slice_get_x_fn slice_get_x;
    model_fn Lprior;
    model_fn Llike;
    log_fn report;
    mix_fn param_mix;
    int K_mix_steps;
    ss_blanker blanker;
    transition_function transition;
    size_t num_u_in_transition;
    ss_coalesced has_coalesced;
} pmc_mavm_setup;

void pm_mavm(pmc_mavm_setup* X, int S_iters, Rand_t seed);
void pmc_mavm(pmc_mavm_setup* X, int S_iters, Rand_t seed);
void pmc_slice_mavm(pmc_mavm_setup* X, int S_iters, Rand_t seed);
void pmc_mh_mh_mavm(pmc_mavm_setup* X, int S_iters, Rand_t seed);
void pmc_slice_mh_mavm(pmc_mavm_setup* X, int S_iters, Rand_t seed);
void pmc_slice_slice_mavm(pmc_mavm_setup* X, int S_iters, Rand_t seed);
void pmc_mh_slice_mavm(pmc_mavm_setup* X, int S_iters, Rand_t seed);

#endif /* #ifndef  _MAVM_CLAMP_H_  */
