#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "slice_sample.h"

double slice_update(double xx, void* stuff, Lpstar_fn fn, double step_size, double* Lp_cur, Rand_t seed)
{
    double log_Px, x_l, x_r, xprime, log_uprime, rr;

    if (Lp_cur == NULL)
        log_Px = fn(xx, stuff);
    else
        log_Px = (*Lp_cur);

    log_uprime = log(Randouble(seed)) + log_Px;

    /* Create a horizontal interval (x_l, x_r) enclosing xx */
    rr = Randouble(seed);
    x_l = xx - rr*step_size;
    x_r = xx + (1-rr)*step_size;
    /* This version doesn't do a stepping out part. */

    /*
     * Inner loop:
     * Propose xprimes and shrink interval until good one found
     */
    while (1) {
        xprime = Randouble(seed)*(x_r - x_l) + x_l;
        log_Px = fn(xprime, stuff);
        if (log_Px > log_uprime) {
            break; /* this is the only way to leave the while loop */
        } else {
            /* Shrink in */
            if (xprime > xx) {
                x_r = xprime;
            } else if (xprime < xx) {
                x_l = xprime;
            } else {
                fprintf(stderr, "BUG DETECTED: Shrunk to current position and still not acceptable.\n");
                abort();
            }
        }
    }
    if (Lp_cur != NULL)
        (*Lp_cur) = log_Px;
    return xprime;
}

