function [ac, err] = autocorr_with_err(xx, lags)
% Simple hack to get autocorrelation with error bands.
% Chunk sequence into 10 chunks and measure mean and standard err of estimates.

xx = xx(:);
S = numel(xx);

M = 10;
ac_array = zeros(numel(lags), M);

start = 1;
for mm = 1:M
    stop = start + floor(S/M) - 1;
    if ~exist('autocorr')
        ac_array(:,mm) = autocorr_diag_rcoda(xx(start:stop), lags);
    else
        % Faster version, requires a Matlab toolbox:
        assert(all(diff(lags) == 1));
        assert((lags(1) == 0) || (lags(1) == 1));
        ac = autocorr(xx(start:stop), lags(end));
        if lags(1) == 1
            ac = ac(2:end);
        end
        ac_array(:, mm) = ac(:);
    end
    start = stop + 1;
end

ac = mean(ac_array, 2);
err = std(ac_array, [], 2)/sqrt(M);

