function corr_plot(xx, names, max_lag, cost_scalings, filename)
%            xx  SxMxK matrix  M runs of S samples, for K different methods
%         names  cell array    K legend names, e.g.,  {'PM', 'APM MI+MH', 'APM SS+SS'}
%       max_lag  1x1
% cost_scalings  1xK           How much to stretch each of the curves. Make the first of these "1.0"
% filename       string        Create filename.eps and filename.pdf

M = size(xx, 2);
K = size(xx, 3);
handles = zeros(1, K);

if (nargin < 4) || isempty(cost_scalings)
    cost_scalings = ones(1, K);
end

clf; hold all;
for kk = 1:K
    ac_array = zeros(max_lag, M);
    for mm = 1:M
        ac = autocorr(xx(:, mm, kk), max_lag);
        ac_array(:, mm) = ac(2:end); % For some reason I've been plotting 1:max_lag, not 0:max_lag
    end
    ac_mu = mean(ac_array, 2);
    ac_err = std(ac_array, [], 2)/sqrt(M);
    handles(kk) = plot_errorband((1:max_lag)*cost_scalings(kk), ac_mu, ac_err);
end
legend(handles, names);
xlim([0 max_lag]);
% May want to play with ylim too.

if (nargin > 3) && ~isempty(filename)
    set(gcf, 'PaperPosition', [1 1 4.5 2.75]*3.5);
    set(gca, 'OuterPosition', [0 0 1 1]);
    set(gca, 'Position', [0.4 0.4 0.5 0.5]);
    print([filename, '.eps'], '-depsc');
    system(['epstopdf ' filename, '.eps']);
end

