function [xx, uu, Lfh] = pm_eslice(log_fhat, xx, prior, uu, type, Lfh, varargin)

% Update auxiliary variables
log_udist = @(u) log_fhat(xx, u, varargin{:});
if isequal(type, 'gaussian') || isequal(type, 'gaussian_small')
    [uu, Lfh] = elliptical_slice(uu, randn(size(uu)), log_udist, Lfh);
elseif isequal(type, 'uniform')
    fixed_initial_brackets = [zeros(numel(uu), 1), ones(numel(uu), 1)];
    [uu, Lfh] = slice_sweep(log_udist, uu, fixed_initial_brackets, 0, Lfh);
elseif isequal(type, 'clamp_gaussian')
    % Assume can only clamp random numbers (pretend don't know they're gaussian)
    u_prop = randn(size(uu));
    Lfh_prop = log_udist(u_prop);
    if log(rand()) < (Lfh_prop - Lfh)
        % accept
        uu = u_prop;
        Lfh = Lfh_prop;
    end
elseif isequal(type, 'bb_gaussian')
    % "Black box": simulate not being able to even clamp what's inside.
    % Generate independent random numbers every time.
    log_xdist = @(x) log_fhat(x, randn(size(uu)), varargin{:});
    global BAIL_OUT;
    BAIL_OUT = true;
else
    error('Unknown type of uu');
end

if ~isequal(type, 'bb_gaussian')
    log_xdist = @(x) log_fhat(x, uu, varargin{:});
end

% Update target variables
randn_ray = randn(size(xx));
direction = randn_ray / sqrt(randn_ray(:)'*randn_ray(:));
log_xdist = @(r) log_fhat(xx + r*direction, uu, varargin{:});
width = 4;
step_out = false;
[r, Lfh] = slice_sweep(log_xdist, 0, width, step_out, Lfh);
xx = xx + r*direction;

BAIL_OUT = [];
