function [xx, uu, Lfh, accept] = pmc_mh_joint(log_fhat, xx, uu, type, widths, Lfh, varargin)

% Version that clamps random numbers between MH updates. Only count acceptances
% on xx, as that's what might monitor to adjust step sizes, and in this version
% there's nothing to adjust in uu updates.

D = numel(xx);
x_prop = xx;

% Propose update for auxiliary variables
if isequal(type, 'gaussian')
    u_prop = randn(size(uu));
elseif isequal(type, 'uniform')
    u_prop = rand(size(uu));
else
    error('Unknown type of uu');
end

Lfh_prop = log_fhat(xx, u_prop, varargin{:});
if log(rand()) < (Lfh_prop - Lfh)
    Lfh = Lfh_prop;
    uu = u_prop;
end

x_prop = xx + widths.*randn(size(xx));

Lfh_prop = log_fhat(x_prop, uu, varargin{:});
if log(rand()) < (Lfh_prop - Lfh)
    accept = 1;
    Lfh = Lfh_prop;
    xx = x_prop;
else
    accept = 0;
end

