function toy_expt_compare(D, S, K)

% % Need to run toy_expt_steps_... first
% % Compare slice sampling to (near) optimal step-size MH methods.

if (nargin < 1) || isempty(D)
    D = 5;
end
if (nargin < 2) || isempty(S)
    S = 1e5;
end
if (nargin < 3) || isempty(K)
    K = 1;
end

if K == 1
    expt = sprintf('D%d', D);
else
    expt = sprintf('D%d_K%d', D, K);
end
ws = load(sprintf('toy_expt_steps_pmc_%s.mat', expt));

optimal_ar = 0.234; % For high dimensions.
[mn, idx] = min(abs(ws.ar_results - optimal_ar));
step_size = ws.step_sizes(idx);
clear ws

log_fhat = @toy_log_fhat;
log_efhat = @toy_log_efhat;
global LOG_EFHAT_TOY_COUNTER;
global LOG_FHAT_TOY_COUNTER;

% PMC-MH
fprintf('PMC-MH\n');
pmc_mh_samples = zeros(S, D);
pmc_mh_accepts = 0;
xx = randn(D, 1);
uu = randn(D*K, 1);
Lfh = log_fhat(xx, uu);
type = 'gaussian';
widths = repmat(step_size, D, 1);
for ss = 1:S
    if ~mod(ss, 1000);
        fprintf('Iteration %d / %d\r', ss, S);
    end
    [xx, uu, Lfh, accepts] = pmc_mh_joint(log_fhat, xx, uu, type, widths, Lfh);
    pmc_mh_samples(ss, :) = xx(:)';
    pmc_mh_accepts = pmc_mh_accepts + accepts;
end
fprintf('\n');
%pmc_mh_ess = effective_size_rcoda(pmc_mh_samples(:, 1)) / S; % ONLY USING FIRST DIM
%pmc_mh_ar = pmc_mh_accepts / (D*S); % If used pmc_mh
pmc_mh_ar = pmc_mh_accepts / S; % If used pmc_mh_joint

max_lag = 2000;
lags = 1:max_lag;
scale_pmc_mh= 2;
[acf_pmc_mh, err_pmc_mh] = autocorr_with_err(pmc_mh_samples(:, 1), lags);
clear pmc_mh_samples

% PM-MH
fprintf('PM-MH\n');
pm_mh_samples = zeros(S, D);
pm_mh_accepts = 0;
xx = randn(D, 1);
uu = randn(D*K, 1);
Lfh = log_fhat(xx, uu);
type = 'gaussian';
widths = repmat(step_size, D, 1);
for ss = 1:S
    if ~mod(ss, 1000);
        fprintf('Iteration %d / %d\r', ss, S);
    end
    [xx, uu, Lfh, accepts] = pm_mh_joint(log_fhat, xx, uu, type, widths, Lfh);
    pm_mh_samples(ss, :) = xx(:)';
    pm_mh_accepts = pm_mh_accepts + accepts;
end
fprintf('\n');
%pm_mh_ess = effective_size_rcoda(pm_mh_samples(:, 1)) / S; % ONLY USING FIRST DIM
%pm_mh_ar = pm_mh_accepts / (D*S); % If used pm_mh
pm_mh_ar = pm_mh_accepts / S; % If used pm_mh_joint

scale_pm= 1;
[acf_pm_mh, err_pm_mh] = autocorr_with_err(pm_mh_samples(:, 1), lags);
clear pm_mh_samples


% PM-ESLICE-MH
fprintf('PMC-ESLICE-MH\n');
LOG_FHAT_TOY_COUNTER = 0;
pmc_eslice_mh_samples = zeros(S, D);
pmc_eslice_mh_accepts = 0;
xx = randn(D, 1);
uu = randn(D*K, 1);
Lfh = log_fhat(xx, uu);
widths = repmat(step_size, D, 1);
for ss = 1:S
    if ~mod(ss, 1000);
        fprintf('Iteration %d / %d\r', ss, S);
    end
    [xx, uu, Lfh, accepts] = pmc_eslice_mh_joint(log_fhat, xx, uu, widths, Lfh);
    pmc_eslice_mh_samples(ss, :) = xx(:)';
    pmc_eslice_mh_accepts = pmc_eslice_mh_accepts + accepts;
end
fprintf('\n');
%pmc_eslice_mh_ess = effective_size_rcoda(pmc_eslice_mh_samples(:, 1)) / S; % ONLY USING FIRST DIM
%pmc_eslice_mh_ar = pmc_eslice_mh_accepts / (D*S); % If used pmc_eslice_mh
pmc_eslice_mh_ar = pmc_eslice_mh_accepts / S; % If used pmc_eslice_mh_joint
pmc_eslice_mh_evals = LOG_FHAT_TOY_COUNTER;

scale_pmc_eslice_mh = pmc_eslice_mh_evals/(S*K);
[acf_pmc_eslice_mh, err_pmc_eslice_mh] = autocorr_with_err(pmc_eslice_mh_samples(:, 1), lags);
clear pmc_eslice_mh_samples

% PM-ESLICE-RSLICE
fprintf('PMC-ESLICE-RSLICE\n');
LOG_FHAT_TOY_COUNTER = 0;
pmc_eslice_rslice_samples = zeros(S, D);
pmc_eslice_rslice_evals = 0;
xx = randn(D, 1);
uu = randn(D*K, 1);
Lfh = log_fhat(xx, uu);
width = 4;
for ss = 1:S
    if ~mod(ss, 1000);
        fprintf('Iteration %d / %d\r', ss, S);
    end
    [xx, uu, Lfh] = pmc_eslice_rslice_joint(log_fhat, xx, uu, width, Lfh);
    pmc_eslice_rslice_samples(ss, :) = xx(:)';
end
fprintf('\n');
%pmc_eslice_rslice_ess = effective_size_rcoda(pmc_eslice_rslice_samples(:, 1)) / S; % ONLY USING FIRST DIM
pmc_eslice_rslice_evals = LOG_FHAT_TOY_COUNTER;

scale_pmc_eslice_rslice = pmc_eslice_rslice_evals/(S*K);
[acf_pmc_eslice_rslice, err_pmc_eslice_rslice] = autocorr_with_err(pmc_eslice_rslice_samples(:, 1), lags);
clear pmc_eslice_rslice_samples

% PMC-RSLICE
fprintf('PMC-RSLICE\n');
type = 'clamp_gaussian'
LOG_FHAT_TOY_COUNTER = 0;
step_out = false;
pmc_rslice_samples = zeros(S, D);
xx = randn(D, 1);
uu = randn(D*K, 1);
Lfh = log_fhat(xx, uu);
for ss = 1:S
    if ~mod(ss, 1000);
        fprintf('Iteration %d / %d\r', ss, S);
    end
    [xx, uu, Lfh] = pm_rslice(log_fhat, xx, randn(size(xx)), uu, type, Lfh);
    pmc_rslice_samples(ss, :) = xx(:)';
end
fprintf('\n');
%pmc_rslice_ess = effective_size_rcoda(pmc_rslice_samples(:, 1)) / S; % ONLY USING FIRST DIM
pmc_rslice_evals = LOG_FHAT_TOY_COUNTER;

scale_pmc_rslice = pmc_rslice_evals/(S*K);
[acf_pmc_rslice, err_pmc_rslice] = autocorr_with_err(pmc_rslice_samples(:, 1), lags);
clear pmc_rslice_samples

save(sprintf('toy_expt_compare_%s', expt));
