function toy_expt_steps_pmc(D, S, step_sizes, K)

if (nargin < 1) || isempty(D)
    D = 5;
end
if (nargin < 2) || isempty(S)
    S = 1e5;
end
if (nargin < 3) || isempty(step_sizes)
    step_sizes = 0.1:0.05:2;
end
if (nargin < 4) || isempty(K)
    K = 1; % Number of samples in estimator
end
num_sizes = length(step_sizes);

log_fhat = @toy_log_fhat;
ess_results = zeros(1, num_sizes);
ar_results = zeros(1, num_sizes);
for ii = 1:num_sizes
    fprintf('Step size: %g    \n', step_sizes(ii));
    mh_samples = zeros(S, D);
    mh_accepts = 0;
    xx = randn(D, 1);
    uu = randn(D*K, 1);
    Lfh = log_fhat(xx, uu);
    type = 'gaussian';
    widths = repmat(step_sizes(ii), D, 1);
    for ss = 1:S
        if ~mod(ss, 1000);
            fprintf('Iteration %d / %d\r', ss, S);
        end
        [xx, uu, Lfh, accepts] = pmc_mh_joint(log_fhat, xx, uu, type, widths, Lfh);
        mh_samples(ss, :) = xx(:)';
        mh_accepts = mh_accepts + accepts;
    end
    fprintf('\n');
    ess_results(ii) = effective_size_rcoda(mh_samples(:, 1)) / S; % ONLY USING FIRST DIM
    %ar_results(ii) = mh_accepts / (D*S);
    ar_results(ii) = mh_accepts / S;
end

if K == 1
    save(sprintf('toy_expt_steps_pmc_D%d', D));
else
    save(sprintf('toy_expt_steps_pmc_D%d_K%d', D, K));
end

