function toy_hist_samples(D, K)

% Gather limited number of samples to look at traces and plot histogram for
% paper

if (nargin < 1) || isempty(D)
    D = 5;
end
if (nargin < 2) || isempty(K)
    K = 1;
end

if K == 1
    expt = sprintf('D%d', D);
else
    expt = sprintf('D%d_K%d', D, K);
end
ws = load(sprintf('toy_expt_steps_pmc_%s.mat', expt));

optimal_ar = 0.234; % For high dimensions.
[mn, idx] = min(abs(ws.ar_results - optimal_ar));
step_size = ws.step_sizes(idx);
clear ws

log_fhat = @toy_log_fhat;
log_efhat = @toy_log_efhat;
global LOG_EFHAT_TOY_COUNTER;
global LOG_FHAT_TOY_COUNTER;

% PMC-MH
S = 50000;
fprintf('PMC-MH\n');
pmc_mh_samples = zeros(S, D);
pmc_mh_accepts = 0;
xx = randn(D, 1);
uu = randn(D*K, 1);
Lfh = log_fhat(xx, uu);
type = 'gaussian';
widths = repmat(step_size, D, 1);
for ss = 1:S
    if ~mod(ss, 1000);
        fprintf('Iteration %d / %d\r', ss, S);
    end
    [xx, uu, Lfh, accepts] = pmc_mh_joint(log_fhat, xx, uu, type, widths, Lfh);
    pmc_mh_samples(ss, :) = xx(:)';
    pmc_mh_accepts = pmc_mh_accepts + accepts;
end
fprintf('\n');
%pmc_mh_ess = effective_size_rcoda(pmc_mh_samples(:, 1)) / S; % ONLY USING FIRST DIM
%pmc_mh_ar = pmc_mh_accepts / (D*S); % If used pmc_mh
pmc_mh_ar = pmc_mh_accepts / S; % If used pmc_mh_joint

% PM-MH
S = 100000;
fprintf('PM-MH\n');
pm_mh_samples = zeros(S, D);
pm_mh_accepts = 0;
xx = randn(D, 1);
uu = randn(D*K, 1);
Lfh = log_fhat(xx, uu);
type = 'gaussian';
widths = repmat(step_size, D, 1);
for ss = 1:S
    if ~mod(ss, 1000);
        fprintf('Iteration %d / %d\r', ss, S);
    end
    [xx, uu, Lfh, accepts] = pm_mh_joint(log_fhat, xx, uu, type, widths, Lfh);
    pm_mh_samples(ss, :) = xx(:)';
    pm_mh_accepts = pm_mh_accepts + accepts;
end
fprintf('\n');
%pm_mh_ess = effective_size_rcoda(pm_mh_samples(:, 1)) / S; % ONLY USING FIRST DIM
%pm_mh_ar = pm_mh_accepts / (D*S); % If used pm_mh
pm_mh_ar = pm_mh_accepts / S; % If used pm_mh_joint

save(sprintf('toy_hist_samples_%s', expt));
